package XML::TreePuller::Constants;

our $VERSION = '0.1.7';

use base qw(Exporter);

our @EXPORT = qw(
	XML_TREEPULLER_ELEMENT_TYPE XML_TREEPULLER_ELEMENT_NAME XML_TREEPULLER_ELEMENT_NAMESPACE
	XML_TREEPULLER_ELEMENT_ATTRIBUTES XML_TREEPULLER_ELEMENT_CHILDREN XML_TREEPULLER_ELEMENT_PARENT
	XML_TREEPULLER_ELEMENT_ROOT XML_TREEPULLER_ELEMENT_DEPTH XML_TREEPULLER_ELEMENT_NEXT_SIBLING
	XML_TREEPULLER_ELEMENT_PREV_SIBLING XML_TREEPULLER_ELEMENT_XPATH_ENGINE
);

use constant XML_TREEPULLER_ELEMENT_TYPE => 0;
use constant XML_TREEPULLER_ELEMENT_NAME => 1;
use constant XML_TREEPULLER_ELEMENT_NAMESPACE => 2;
use constant XML_TREEPULLER_ELEMENT_ATTRIBUTES => 3;
use constant XML_TREEPULLER_ELEMENT_CHILDREN => 4;
use constant XML_TREEPULLER_ELEMENT_PARENT => 5;
use constant XML_TREEPULLER_ELEMENT_ROOT => 6;
use constant XML_TREEPULLER_ELEMENT_DEPTH => 7;
use constant XML_TREEPULLER_ELEMENT_NEXT_SIBLING => 8;
use constant XML_TREEPULLER_ELEMENT_PREV_SIBLING => 9;
use constant XML_TREEPULLER_ELEMENT_XPATH_ENGINE => 10;

1;